// JC Wed Mar  3 10:20:52 CET 2004 - pocitani energie. 

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string>
#include <map>

#ifdef _WINDOWS
	#include "wfsource.h"
	#include "wspecgraf.h"
	#include <io.h>
#else
	#include <unistd.h>
	#include <sys/ioctl.h>
	#include <sys/types.h>
	#include <sys/stat.h>
	#include "lwfsource.h"
	#include "specgraf.h"
#endif 

#include "config.h"
#include "dspc.h"
#include "lpc.h"
#include "filelen.h"


int main(int argc, char *argv[])
{
	try
	{
		// true - read from sound card, false - read from a file 
		bool online = false;
		char *ifile = "test.l16";
		char *ofile = "out";

		// Input device, graph
		#ifdef _WINDOWS
			WFSource WFS;
			//WSpecGraf SG;
		#else
			LWFSource WFS;
			WFS.setDevice("/dev/dsp");
			//SpecGraf SG;
		#endif
		WFS.setFormat(8000, 1, 16);
		if(online)
		    WFS.open();

		// Features
		LPC Lpc;	
	
		// Output graph
		//     posY, posX, dx, dy, nbins, min_val, max_val
		// puvodne bylo SG.set(0, 0, -1, -1, BANKS, 0.0f, 50000.0f);
		//SG.set(0, 0, -1, -1, ENE_BUF_SIZE, 0.0f, 75000.0f);

		short *sgn;
		float *sgn_fl;
		float features[VECTOR_SIZE];
		FILE *fp_in, *fp_out;  
		
		// Input file and buffers
		if(online)
		{
		    sgn = new short[VECTOR_SIZE];
		    sgn_fl = new float[VECTOR_SIZE];
		}  
		else
		{
		    fp_in = fopen(ifile, "rb");
		    if(!fp_in)
		    {
			fprintf(stderr, "Can not open the file: %s\n", ifile);
			exit(1);
		    }
		    sgn = new short[filelen(fp_in)];
		    sgn_fl = new float[filelen(fp_in)];		    
		}
		
		// Output file
		fp_out = fopen(ofile, "wb");
		if(!fp_out)
		{
		    fprintf(stderr, "Can not create the file: %s\n", ofile);
		    exit(2);
		}


		int frame = 0;
		
		do
		{
			int i;

			if(online)
			{
			    WFS.read((char *)sgn, VECTOR_SIZE * sizeof(short));
			    
			    // shorts -> floats
	    		    for(i = 0; i < VECTOR_SIZE; i++)
				sgn_fl[i] = (float)sgn[i];
			    
			    Lpc.addWaveform(sgn_fl, VECTOR_SIZE);
			}
			else
			{
			    read(fileno(fp_in), (char *)sgn, filelen(fp_in));
			
			    // shorts -> floats
			    for(i = 0; i < filelen(fp_in) / 2; i++)
				sgn_fl[i] = (float)sgn[i];

			    Lpc.addWaveform(sgn_fl, filelen(fp_in) / 2);
			}
			    
			
			while(Lpc.getFeatures(features))
			{        
			    printf("%d ", frame);
			    frame++;
			    int j;
			    for(j = 0; j < 12; j++)
				printf(" %6.3f", features[j]);
			    printf("\n");
			    
			    fwrite(features, sizeof(float), 12, fp_out); 
			}
		} while(online);
		
		if(online)
		    WFS.close();
		else
		    fclose(fp_in);
		    
		fclose(fp_out);
		
		delete [] sgn;
		delete [] sgn_fl;
	}
	catch(wf_error &er)
	{
		fprintf(stderr, "Sound recording error: %s\n", er.what());
		exit(1);
	}
	return 0;
}
